unit uFlyweightFactory;

interface

uses
  Classes, uFlyweight, uConcreteFlyweight;

type
  TFlyweightFactory = class
  private
    // varivel para armazenar os objetos compartilhados
    ListaCartoes: TStringList;
  public
    constructor Create;
    destructor Destroy; override;

    // mtodo principal do Flyweight,
    // responsvel por encontrar e retornar o objeto j criado
    // ou cri-lo caso no exista, adicionando-o na lista de objetos compartilhados
    function GetCartao(const Pais: string): TCartao;
  end;

implementation

uses
  SysUtils;

{ TFlyweightFactory }

constructor TFlyweightFactory.Create;
begin
  // cria a lista de objetos compartilhados
  ListaCartoes := TStringList.Create;
  ListaCartoes.Sorted := True;
end;

destructor TFlyweightFactory.Destroy;
var
  Contador: integer;
begin
  for Contador := 0 to Pred(ListaCartoes.Count) do
    ListaCartoes.Objects[Contador].Free;

  // libera a lista de objetos compartilhados da memria
  FreeAndNil(ListaCartoes);
  inherited;
end;

function TFlyweightFactory.GetCartao(const Pais: string): TCartao;
var
  Indice: integer;
begin
  result := nil;

  // tenta encontrar o objeto compartilhado atravs do nome do pas
  if ListaCartoes.Find(Pais, Indice) then
  begin
    // caso seja encontrado, o objeto compartilhado  retornado
    result := (ListaCartoes.Objects[Indice]) as TCartao;
    Exit;
  end;

  // caso no seja encontrado,  criado...
  if Pais = 'Espanha' then
    result := TCartaoEspanha.Create
  else if Pais = 'EUA' then
    result := TCartaoEUA.Create
  else if Pais = 'Brasil' then
    result := TCartaoBrasil.Create
  else if Pais = 'Itlia' then
    result := TCartaoItalia.Create;

  // ... e depois adicionado na lista de objetos compartilhados
  // para que no precise ser criado novamente nas prximas iteraes
  ListaCartoes.AddObject(Pais, result);
end;

end.
